# Copyright (C) 2023 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
#
"""mark4 workflow renaming

Revision ID: 91091612b6d0
Revises: d20ceed949b3
Create Date: 2023-09-22 13:34:54.054114

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "91091612b6d0"
down_revision = "d20ceed949b3"
branch_labels = None
depends_on = None


old_ingest_mk_four_obs_sh = """#!/bin/sh
set -o errexit

SBIN_PATH=/lustre/aoc/cluster/pipeline/$CAPO_PROFILE/workspaces/sbin

# Get NGAS hosts and set up variables to randomly select one
NGASHOSTSTR=$($SBIN_PATH/pycapo -q archive-ingestion.NGASHosts)
NGASHOSTARR=(`/bin/echo ${NGASHOSTSTR}`) # Put the space-delimited host list into an array
NGASHOSTLEN=${#NGASHOSTARR[@]}

# Copy from the difx area to the Workspaces staging area
WSSTAGINGDIR=$($SBIN_PATH/pycapo -q edu.nrao.workspaces.IngestionSettings.stagingDirectory)
/bin/cp -r $1 $WSSTAGINGDIR

OBSDIR=$(/bin/basename $1)
/bin/chmod -R 750 $WSSTAGINGDIR/$OBSDIR # Make sure NGAS has permissions to ingest the files
cd $WSSTAGINGDIR/$OBSDIR

for FILE in *; do
    # Pick random NGAS host to distribute the ingestion load
    NGASINDEX=$(($RANDOM % $NGASHOSTLEN))
    NGASHOST=${NGASHOSTARR[$NGASINDEX]}

    FULLPATH=$(/bin/readlink -f $FILE)
    NGASCMD="${NGASHOST}ARCHIVE?filename=file://${FULLPATH}"

    INGESTOUT=$(/bin/curl $NGASCMD)
    if echo $INGESTOUT | grep -i "error"; then
        echo "Failed to ingest ${FILE}"
        exit 1
    fi
done"""

# Workflow will rename the directory to indicate the ingestion status
new_ingest_mk_four_obs_sh = """#!/bin/sh

ingested=$1.ingested
failed=$1.failed

SBIN_PATH=/lustre/aoc/cluster/pipeline/$CAPO_PROFILE/workspaces/sbin

# Are we running live NGAS ingestion or testing only? check what we\\'re feeding the manifests
ISLIVE=$($SBIN_PATH/pycapo -q archive-ingestion.ingestNGAS)

# Get NGAS hosts and set up variables to randomly select one
NGASHOSTSTR=$($SBIN_PATH/pycapo -q archive-ingestion.NGASHosts)
NGASHOSTARR=(`/bin/echo ${NGASHOSTSTR}`) # Put the space-delimited host list into an array
NGASHOSTLEN=${#NGASHOSTARR[@]}

# Copy from the difx area to the Workspaces staging area
WSSTAGINGDIR=$($SBIN_PATH/pycapo -q edu.nrao.workspaces.IngestionSettings.stagingDirectory)
/bin/cp -r $1 $WSSTAGINGDIR

OBSDIR=$(/bin/basename $1)
/bin/chmod -R 750 $WSSTAGINGDIR/$OBSDIR # Make sure NGAS has permissions to ingest the files
cd $WSSTAGINGDIR/$OBSDIR

for FILE in *; do
    # Pick random NGAS host to distribute the ingestion load
    NGASINDEX=$(($RANDOM % $NGASHOSTLEN))
    NGASHOST=${NGASHOSTARR[$NGASINDEX]}

    FULLPATH=$(/bin/readlink -f $FILE)
    NGASCMD="${NGASHOST}ARCHIVE?filename=file://${FULLPATH}"

    if [[ $ISLIVE = true ]]; then
        INGESTOUT=$(/bin/curl $NGASCMD)
        if echo $INGESTOUT | grep -i "error"; then
            echo "Failed to ingest ${FILE}"
            mv $1 $failed
            exit 1
        fi
    else
        echo "Test Only! ${NGASCMD}"
    fi
done

mv $1 $ingested
"""


def upgrade():
    op.execute(
        f"""
        UPDATE workflow_templates
        SET content=E'{new_ingest_mk_four_obs_sh}'
        WHERE filename='ingest_mk_four_obs.sh'
        """
    )


def downgrade():
    op.execute(
        f"""
        UPDATE workflow_templates
        SET content=E'{old_ingest_mk_four_obs_sh}'
        WHERE filename='ingest_mk_four_obs.sh'
        """
    )
